@@ -1,15 +1,17 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
+"""base_component."""
 
 import abc
-import uuid
 import datetime
+import uuid
 from enum import IntEnum
+
 from .base_task import BaseTaskState
 
 
 class BaseComponentState(IntEnum):
-    """BaseComponentState"""
+    """BaseComponentState."""
 
     NONE = 0
     READY = 1
@@ -19,7 +21,8 @@ class BaseComponentState(IntEnum):
 
 
 class BaseComponent(object, metaclass=abc.ABCMeta):
-    """BaseComponent
+    """BaseComponent.
+
     BaseComponent class for expressing target product.
     This class can be used as template.
 
@@ -79,7 +82,7 @@ class BaseComponent(object, metaclass=abc.ABCMeta):
         placed_workplace=None,
         placed_workplace_id_record=None,
     ):
-
+        """init."""
         # ----
         # Constraint parameter on simulation
         # --
@@ -129,7 +132,7 @@ class BaseComponent(object, metaclass=abc.ABCMeta):
 
     def extend_child_component_list(self, child_component_list):
         """
-        Extend the list of child components
+        Extend the list of child components.
 
         Args:
             child_component_list (List[BaseComponent]):
@@ -188,6 +191,7 @@ class BaseComponent(object, metaclass=abc.ABCMeta):
     def is_ready(self):
         """
         Check READY component or not.
+
         READY component is defined by satisfying the following conditions:
 
           - All tasks are not NONE.
@@ -263,6 +267,7 @@ class BaseComponent(object, metaclass=abc.ABCMeta):
     def initialize(self, state_info=True, log_info=True, check_task_state=True):
         """
         Initialize the following changeable basic variables of BaseComponent.
+
         If `state_info` is True, the following attributes are initialized.
 
           - `state`
@@ -295,17 +300,20 @@ class BaseComponent(object, metaclass=abc.ABCMeta):
             if check_task_state:
                 self.check_state()
 
-    def reverse_log_information(self):
-        """
-        Reverse log information of all.
-        """
+    def reverse_log_information(self, delete_head=False):
+        """Reverse log information of all."""
         self.state_record_list = self.state_record_list[::-1]
         self.placed_workplace_id_record = self.placed_workplace_id_record[::-1]
+        if delete_head:
+            self.state_record_list.pop(0)
+            # cost_head = self.state_record_list.pop(0)
+            # self.state_record_list.append(cost_head)  # insert
+            self.placed_workplace_id_record.pop(0)
+            # log_head = self.placed_workplace_id_record.pop(0)
+            # self.placed_workplace_id_record.append(log_head)  # insert
 
     def check_state(self):
-        """
-        Check and update the `state` of this component.
-        """
+        """Check and update the `state` of this component."""
         self.__check_ready()
         self.__check_working()
         self.__check_finished()
@@ -340,22 +348,19 @@ class BaseComponent(object, metaclass=abc.ABCMeta):
             self.state = BaseComponentState.FINISHED
 
     def record_placed_workplace_id(self):
-        """
-        Record workplace id in this time to `placed_workplace_id_record`.
-        """
+        """Record workplace id in this time to `placed_workplace_id_record`."""
         record = None
         if self.placed_workplace is not None:
             record = self.placed_workplace.ID
         self.placed_workplace_id_record.append(record)
 
     def record_state(self):
-        """
-        Record current `state` in `state_record_list`
-        """
+        """Record current `state` in `state_record_list`."""
         self.state_record_list.append(self.state)
 
     def __str__(self):
-        """
+        """str.
+
         Returns:
             str: name of BaseComponent
         Examples:
@@ -433,20 +438,16 @@ class BaseComponent(object, metaclass=abc.ABCMeta):
                     if state == BaseComponentState.WORKING:
                         if previous_state == BaseComponentState.READY:
                             ready_time_list.append(
-                                (from_time, (to_time - 1) - from_time + finish_margin)
+                                (from_time, to_time - from_time + finish_margin)
                             )
                     from_time = time
                     to_time = -1
             previous_state = state
 
             if previous_state == BaseComponentState.WORKING:
-                working_time_list.append(
-                    (from_time, time - 1 - from_time + finish_margin)
-                )
+                working_time_list.append((from_time, time - from_time + finish_margin))
             elif previous_state == BaseComponentState.READY:
-                ready_time_list.append(
-                    (from_time, time - 1 - from_time + finish_margin)
-                )
+                ready_time_list.append((from_time, time - from_time + finish_margin))
         return ready_time_list, working_time_list
 
     def create_data_for_gantt_plotly(
@@ -457,8 +458,9 @@ class BaseComponent(object, metaclass=abc.ABCMeta):
         view_ready=False,
     ):
         """
-        Create data for gantt plotly from start_time_list and finish_time_list
-        of BaseTask in targeted_task_list.
+        Create data for gantt plotly.
+
+        From start_time_list and finish_time_list of BaseTask in targeted_task_list.
 
         Args:
             init_datetime (datetime.datetime):
@@ -486,31 +488,31 @@ class BaseComponent(object, metaclass=abc.ABCMeta):
             for (from_time, length) in ready_time_list:
                 to_time = from_time + length
                 df.append(
-                    dict(
-                        Task=self.name,
-                        Start=(init_datetime + from_time * unit_timedelta).strftime(
+                    {
+                        "Task": self.name,
+                        "Start": (init_datetime + from_time * unit_timedelta).strftime(
                             "%Y-%m-%d %H:%M:%S"
                         ),
-                        Finish=(init_datetime + to_time * unit_timedelta).strftime(
+                        "Finish": (init_datetime + to_time * unit_timedelta).strftime(
                             "%Y-%m-%d %H:%M:%S"
                         ),
-                        State="READY",
-                        Type="Component",
-                    )
+                        "State": "READY",
+                        "Type": "Component",
+                    }
                 )
         for (from_time, length) in working_time_list:
             to_time = from_time + length
             df.append(
-                dict(
-                    Task=self.name,
-                    Start=(init_datetime + from_time * unit_timedelta).strftime(
+                {
+                    "Task": self.name,
+                    "Start": (init_datetime + from_time * unit_timedelta).strftime(
                         "%Y-%m-%d %H:%M:%S"
                     ),
-                    Finish=(init_datetime + to_time * unit_timedelta).strftime(
+                    "Finish": (init_datetime + to_time * unit_timedelta).strftime(
                         "%Y-%m-%d %H:%M:%S"
                     ),
-                    State="WORKING",
-                    Type="Component",
-                )
+                    "State": "WORKING",
+                    "Type": "Component",
+                }
             )
         return df
