@@ -1,22 +1,28 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
+"""base_organization."""
 
 import abc
-from typing import List
-from .base_team import BaseTeam
-from .base_workplace import BaseWorkplace
-from .base_worker import BaseWorker, BaseWorkerState
-from .base_facility import BaseFacility, BaseFacilityState
-import plotly.figure_factory as ff
-import networkx as nx
-import plotly.graph_objects as go
 import datetime
-import matplotlib.pyplot as plt
 import warnings
+from typing import List
+
+import matplotlib.pyplot as plt
+
+import networkx as nx
+
+import plotly.figure_factory as ff
+import plotly.graph_objects as go
+
+from .base_facility import BaseFacility, BaseFacilityState
+from .base_team import BaseTeam
+from .base_worker import BaseWorker, BaseWorkerState
+from .base_workplace import BaseWorkplace
 
 
 class BaseOrganization(object, metaclass=abc.ABCMeta):
-    """BaseOrganization
+    """BaseOrganization.
+
     BaseOrganization class for expressing organization in target project.
     BaseOrganization is consist of multiple BaseTeam and BaseWorkplace.
     This class will be used as template.
@@ -43,6 +49,7 @@ class BaseOrganization(object, metaclass=abc.ABCMeta):
         # Basic variables
         cost_list=None,
     ):
+        """init."""
         # ----
         # Constraint parameters on simulation
         # --
@@ -61,7 +68,8 @@ class BaseOrganization(object, metaclass=abc.ABCMeta):
         # Advanced variables for customized simulation
 
     def __str__(self):
-        """
+        """str.
+
         Returns:
             str: name list of BaseTeam
         Examples:
@@ -179,6 +187,7 @@ class BaseOrganization(object, metaclass=abc.ABCMeta):
     ):
         """
         Get team list by using search conditions related to BaseTeam parameter.
+
         If there is no searching condition, this function returns all self.team_list
 
         Args:
@@ -235,6 +244,7 @@ class BaseOrganization(object, metaclass=abc.ABCMeta):
     ):
         """
         Get workplace list by using search conditions related to BaseTeam parameter.
+
         If there is no searching condition, this function returns all self.workplace_list
 
         Args:
@@ -330,6 +340,7 @@ class BaseOrganization(object, metaclass=abc.ABCMeta):
     ):
         """
         Get worker list by using search conditions related to BaseWorker parameter.
+
         This method just executes BaseTeam.get_worker_list() in self.team_list.
 
         Args:
@@ -407,6 +418,7 @@ class BaseOrganization(object, metaclass=abc.ABCMeta):
     ):
         """
         Get facility list by using search conditions related to BaseFacility parameter.
+
         This method just executes BaseTeam.get_facility_list() in self.workplace_list.
 
         Args:
@@ -466,7 +478,7 @@ class BaseOrganization(object, metaclass=abc.ABCMeta):
 
     def initialize(self, state_info=True, log_info=True):
         """
-        Initialize the changeable variables of BaseOrganization
+        Initialize the changeable variables of BaseOrganization.
 
         If `log_info` is True, the following attributes are initialized.
           - cost_list
@@ -488,15 +500,17 @@ class BaseOrganization(object, metaclass=abc.ABCMeta):
         for workplace in self.workplace_list:
             workplace.initialize(state_info=state_info, log_info=log_info)
 
-    def reverse_log_information(self):
-        """
-        Reverse log information of all.
-        """
+    def reverse_log_information(self, delete_head=False):
+        """Reverse log information of all."""
         self.cost_list = self.cost_list[::-1]
         for team in self.team_list:
-            team.reverse_log_information()
+            team.reverse_log_information(delete_head)
         for workplace in self.workplace_list:
-            workplace.reverse_log_information()
+            workplace.reverse_log_information(delete_head)
+        if delete_head:
+            self.cost_list.pop(0)
+            # cost_head = self.cost_list.pop(0)
+            # self.cost_list.append(cost_head)  # insert
 
     def add_labor_cost(
         self,
@@ -539,9 +553,7 @@ class BaseOrganization(object, metaclass=abc.ABCMeta):
         return cost_this_time
 
     def record(self):
-        """
-        Record assigned task id and component.
-        """
+        """Record assigned task id and component."""
         for team in self.team_list:
             team.record_assigned_task_id()
             team.record_all_worker_state()
@@ -550,7 +562,7 @@ class BaseOrganization(object, metaclass=abc.ABCMeta):
             workplace.record_placed_component_id()
             workplace.record_all_facility_state()
 
-    def create_simple_gantt(
+    def plot_simple_gantt(
         self,
         finish_margin=1.0,
         view_ready=False,
@@ -566,8 +578,8 @@ class BaseOrganization(object, metaclass=abc.ABCMeta):
         save_fig_path=None,
     ):
         """
+        Plot Gantt chart by matplotlib.
 
-        Method for creating Gantt chart by matplotlib.
         In this Gantt chart, datetime information is not included.
         This method will be used after simulation.
 
@@ -608,7 +620,82 @@ class BaseOrganization(object, metaclass=abc.ABCMeta):
 
         Returns:
             fig: fig in plt.subplots()
-            gnt: gnt in plt.subplots()
+        """
+        fig, gnt = self.create_simple_gantt(
+            finish_margin=finish_margin,
+            view_ready=view_ready,
+            view_workers=view_workers,
+            view_facilities=view_facilities,
+            team_color=team_color,
+            worker_color=worker_color,
+            workplace_color=workplace_color,
+            facility_color=facility_color,
+            ready_color=ready_color,
+            figsize=figsize,
+            dpi=dpi,
+            save_fig_path=save_fig_path,
+        )
+        return fig
+
+    def create_simple_gantt(
+        self,
+        finish_margin=1.0,
+        view_ready=False,
+        view_workers=True,
+        view_facilities=True,
+        team_color="#0099FF",
+        worker_color="#D9E5FF",
+        workplace_color="#0099FF",
+        facility_color="#D9E5FF",
+        ready_color="#C0C0C0",
+        figsize=[6.4, 4.8],
+        dpi=100.0,
+        save_fig_path=None,
+    ):
+        """
+        Create Gantt chart by matplotlib.
+
+        In this Gantt chart, datetime information is not included.
+        This method will be used after simulation.
+
+        Args:
+            finish_margin (float, optional):
+                Margin of finish time in Gantt chart.
+                Defaults to 1.0.
+            view_ready (bool, optional):
+                View READY time or not.
+                Defaults to False.
+            view_workers (bool, optional):
+                Including workers in networkx graph or not.
+                Default to Trstate = w.state_record_list[time]ue.
+            team_color (str, optional):
+                Node color setting information.
+                Defaults to "#0099FF".
+            worker_color (str, optional):
+                Node color setting information.
+                Defaults to "#D9E5FF".
+            workplace_color (str, optional):
+                Node color setting information.
+                Defaults to "#0099FF".
+            facility_color (str, optional):
+                Node color setting information.
+                Defaults to "#D9E5FF".
+            ready_color (str, optional):
+                Ready Worker/Facility color setting information.
+                Defaults to "#C0C0C0".
+            figsize ((float, float), optional):
+                Width, height in inches.
+                Default to [6.4, 4.8]
+            dpi (float, optional):
+                The resolution of the figure in dots-per-inch.
+                Default to 100.0
+            save_fig_path (str, optional):
+                Path of saving figure.
+                Defaults to None.
+
+        Returns:
+            fig: fig in plt.subplots()
+            gnt: ax in plt.subplots()
         """
         fig, gnt = plt.subplots()
         fig.figsize = figsize
@@ -739,7 +826,8 @@ class BaseOrganization(object, metaclass=abc.ABCMeta):
         save_fig_path=None,
     ):
         """
-        Method for creating Gantt chart by plotly.
+        Create Gantt chart by plotly.
+
         This method will be used after simulation.
 
         Args:
@@ -788,7 +876,7 @@ class BaseOrganization(object, metaclass=abc.ABCMeta):
         colors = (
             colors
             if colors is not None
-            else dict(WORKING="rgb(46, 137, 205)", READY="rgb(107, 127, 135)")
+            else {"WORKING": "rgb(46, 137, 205)", "READY": "rgb(107, 127, 135)"}
         )
         index_col = index_col if index_col is not None else "State"
         df = self.create_data_for_gantt_plotly(
@@ -870,7 +958,8 @@ class BaseOrganization(object, metaclass=abc.ABCMeta):
         save_fig_path=None,
     ):
         """
-        Method for creating cost chart by plotly.
+        Create cost chart by plotly.
+
         This method will be used after simulation.
 
         Args:
@@ -988,7 +1077,7 @@ class BaseOrganization(object, metaclass=abc.ABCMeta):
         **kwds,
     ):
         """
-        Draw networkx
+        Draw networkx.
 
         Args:
             G (networkx.Digraph, optional):
@@ -1032,7 +1121,6 @@ class BaseOrganization(object, metaclass=abc.ABCMeta):
         Returns:
             figure: Figure for a network
         """
-
         fig = plt.figure(figsize=figsize, dpi=dpi)
         G = (
             G
@@ -1148,7 +1236,6 @@ class BaseOrganization(object, metaclass=abc.ABCMeta):
             facility_node_trace: Facility Node information of plotly network.
             edge_trace: Edge information of plotly network.
         """
-
         G = (
             G
             if G is not None
@@ -1164,10 +1251,10 @@ class BaseOrganization(object, metaclass=abc.ABCMeta):
             text=[],
             mode="markers",
             hoverinfo="text",
-            marker=dict(
-                color=team_node_color,
-                size=node_size,
-            ),
+            marker={
+                "color": team_node_color,
+                "size": node_size,
+            },
         )
         worker_node_trace = go.Scatter(
             x=[],
@@ -1175,10 +1262,10 @@ class BaseOrganization(object, metaclass=abc.ABCMeta):
             text=[],
             mode="markers",
             hoverinfo="text",
-            marker=dict(
-                color=worker_node_color,
-                size=node_size,
-            ),
+            marker={
+                "color": worker_node_color,
+                "size": node_size,
+            },
         )
         workplace_node_trace = go.Scatter(
             x=[],
@@ -1186,10 +1273,10 @@ class BaseOrganization(object, metaclass=abc.ABCMeta):
             text=[],
             mode="markers",
             hoverinfo="text",
-            marker=dict(
-                color=workplace_node_color,
-                size=node_size,
-            ),
+            marker={
+                "color": workplace_node_color,
+                "size": node_size,
+            },
         )
         facility_node_trace = go.Scatter(
             x=[],
@@ -1197,10 +1284,10 @@ class BaseOrganization(object, metaclass=abc.ABCMeta):
             text=[],
             mode="markers",
             hoverinfo="text",
-            marker=dict(
-                color=facility_node_color,
-                size=node_size,
-            ),
+            marker={
+                "color": facility_node_color,
+                "size": node_size,
+            },
         )
 
         for node in G.nodes:
@@ -1223,7 +1310,11 @@ class BaseOrganization(object, metaclass=abc.ABCMeta):
                 worker_node_trace["text"] = worker_node_trace["text"] + (node,)
 
         edge_trace = go.Scatter(
-            x=[], y=[], line=dict(width=1, color="#888"), hoverinfo="none", mode="lines"
+            x=[],
+            y=[],
+            line={"width": 1, "color": "#888"},
+            hoverinfo="none",
+            mode="lines",
         )
 
         for edge in G.edges:
@@ -1257,7 +1348,7 @@ class BaseOrganization(object, metaclass=abc.ABCMeta):
         save_fig_path=None,
     ):
         """
-        Draw plotly network
+        Draw plotly network.
 
         Args:
             G (networkx.Digraph, optional):
@@ -1340,22 +1431,22 @@ class BaseOrganization(object, metaclass=abc.ABCMeta):
                 #         hovermode='closest',
                 #         margin=dict(b=20,l=5,r=5,t=40),
                 annotations=[
-                    dict(
-                        ax=edge_trace["x"][index * 2],
-                        ay=edge_trace["y"][index * 2],
-                        axref="x",
-                        ayref="y",
-                        x=edge_trace["x"][index * 2 + 1],
-                        y=edge_trace["y"][index * 2 + 1],
-                        xref="x",
-                        yref="y",
-                        showarrow=True,
-                        arrowhead=5,
-                    )
+                    {
+                        "ax": edge_trace["x"][index * 2],
+                        "ay": edge_trace["y"][index * 2],
+                        "axref": "x",
+                        "ayref": "y",
+                        "x": edge_trace["x"][index * 2 + 1],
+                        "y": edge_trace["y"][index * 2 + 1],
+                        "xref": "x",
+                        "yref": "y",
+                        "showarrow": True,
+                        "arrowhead": 5,
+                    }
                     for index in range(0, int(len(edge_trace["x"]) / 2))
                 ],
-                xaxis=dict(showgrid=False, zeroline=False, showticklabels=False),
-                yaxis=dict(showgrid=False, zeroline=False, showticklabels=False),
+                xaxis={"showgrid": False, "zeroline": False, "showticklabels": False},
+                yaxis={"showgrid": False, "zeroline": False, "showticklabels": False},
             ),
         )
         if save_fig_path is not None:
