@@ -1,20 +1,26 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
+"""base_product."""
 
 import abc
+import datetime
+import warnings
 from typing import List
+
+import matplotlib.pyplot as plt
+
+import networkx as nx
+
+import plotly.figure_factory as ff
+import plotly.graph_objects as go
+
 from .base_component import BaseComponent, BaseComponentState
 from .base_task import BaseTaskState
-import plotly.figure_factory as ff
-import networkx as nx
-import plotly.graph_objects as go
-import datetime
-import matplotlib.pyplot as plt
-import warnings
 
 
 class BaseProduct(object, metaclass=abc.ABCMeta):
-    """BaseProduct
+    """BaseProduct.
+
     BaseProduct class for expressing target product in a project.
     BaseProduct is consist of multiple BaseComponents.
     This class will be used as template.
@@ -25,6 +31,7 @@ class BaseProduct(object, metaclass=abc.ABCMeta):
     """
 
     def __init__(self, component_list: List[BaseComponent]):
+        """init."""
         # ----
         # Constraint parameters on simulation
         # --
@@ -49,7 +56,8 @@ class BaseProduct(object, metaclass=abc.ABCMeta):
             c.initialize(state_info=state_info, log_info=log_info)
 
     def __str__(self):
-        """
+        """str.
+
         Returns:
             str: name list of BaseComponent
         Examples:
@@ -191,12 +199,10 @@ class BaseProduct(object, metaclass=abc.ABCMeta):
                 component_list.append(component)
         return component_list
 
-    def reverse_log_information(self):
-        """
-        Reverse log information of all.
-        """
+    def reverse_log_information(self, delete_head=False):
+        """Reverse log information of all."""
         for c in self.component_list:
-            c.reverse_log_information()
+            c.reverse_log_information(delete_head)
 
     def get_component_list(
         self,
@@ -211,6 +217,7 @@ class BaseProduct(object, metaclass=abc.ABCMeta):
     ):
         """
         Get component list by using search conditions related to BaseComponent parameter.
+
         If there is no searching condition, this function returns `component_list`.
 
         Args:
@@ -298,24 +305,24 @@ class BaseProduct(object, metaclass=abc.ABCMeta):
         return component_list
 
     def record(self):
-        """
-        Record placed workplace id in this time.
-        """
+        """Record placed workplace id in this time."""
         for c in self.component_list:
             c.record_placed_workplace_id()
             c.record_state()
 
     def check_state(self):
-        """
-        Check state
-        """
+        """Check state."""
         for c in self.component_list:
             c.check_state()
 
-    def check_removing_placed_workplace(self, print_debug=False, log_txt=[]):
+    def check_removing_placed_workplace(self, print_debug=False):
         """
         Check removing this product from placed_workplace or not.
+
         If all tasks of this product is finished, this product will be removed automatically.
+
+        Returns:
+            log_txt: List of log text about removing placed workplace.
         """
         top_component_list = list(
             filter(lambda c: len(c.parent_component_list) == 0, self.component_list)
@@ -332,6 +339,7 @@ class BaseProduct(object, metaclass=abc.ABCMeta):
             if all_finished_flag and c.placed_workplace is not None:
                 removing_placed_workplace_component.append(c)
 
+        log_txt = []
         for c in removing_placed_workplace_component:
             if print_debug:
                 print(
@@ -345,8 +353,9 @@ class BaseProduct(object, metaclass=abc.ABCMeta):
             )
             c.placed_workplace.remove_placed_component(c)
             c.set_placed_workplace(None)
+        return log_txt
 
-    def create_simple_gantt(
+    def plot_simple_gantt(
         self,
         finish_margin=1.0,
         view_ready=True,
@@ -357,7 +366,8 @@ class BaseProduct(object, metaclass=abc.ABCMeta):
         save_fig_path=None,
     ):
         """
-        Method for creating Gantt chart by matplotlib.
+        Plot Gantt chart by matplotlib.
+
         In this Gantt chart, datetime information is not included.
         This method will be used after simulation.
 
@@ -386,7 +396,60 @@ class BaseProduct(object, metaclass=abc.ABCMeta):
 
         Returns:
             fig: fig in plt.subplots()
-            gnt: gnt in plt.subplots()
+        """
+        fig, gnt = self.create_simple_gantt(
+            finish_margin=finish_margin,
+            view_ready=view_ready,
+            component_color=component_color,
+            ready_color=ready_color,
+            figsize=figsize,
+            dpi=dpi,
+            save_fig_path=save_fig_path,
+        )
+        return fig
+
+    def create_simple_gantt(
+        self,
+        finish_margin=1.0,
+        view_ready=True,
+        component_color="#FF6600",
+        ready_color="#C0C0C0",
+        figsize=[6.4, 4.8],
+        dpi=100.0,
+        save_fig_path=None,
+    ):
+        """
+        Create Gantt chart by matplotlib.
+
+        In this Gantt chart, datetime information is not included.
+        This method will be used after simulation.
+
+        Args:
+            finish_margin (float, optional):
+                Margin of finish time in Gantt chart.
+                Defaults to 1.0.
+            view_ready (bool, optional):
+                View READY time or not.
+                Defaults to True.
+            component_color (str, optional):
+                Component color setting information.
+                Defaults to "#FF6600".
+            ready_color (str, optional):
+                Ready color setting information.
+                Defaults to "#C0C0C0".
+            figsize ((float, float), optional):
+                Width, height in inches.
+                Default to [6.4, 4.8]
+            dpi (float, optional):
+                The resolution of the figure in dots-per-inch.
+                Default to 100.0
+            save_fig_path (str, optional):
+                Path of saving figure.
+                Defaults to None.
+
+        Returns:
+            fig: fig in plt.subplots()
+            gnt: ax in plt.subplots()
         """
         fig, gnt = plt.subplots()
         fig.figsize = figsize
@@ -471,7 +534,8 @@ class BaseProduct(object, metaclass=abc.ABCMeta):
         save_fig_path=None,
     ):
         """
-        Method for creating Gantt chart by plotly.
+        Create Gantt chart by plotly.
+
         This method will be used after simulation.
 
         Args:
@@ -520,7 +584,7 @@ class BaseProduct(object, metaclass=abc.ABCMeta):
         colors = (
             colors
             if colors is not None
-            else dict(WORKING="rgb(246, 37, 105)", READY="rgb(107, 127, 135)")
+            else {"WORKING": "rgb(246, 37, 105)", "READY": "rgb(107, 127, 135)"}
         )
         index_col = index_col if index_col is not None else "State"
         df = self.create_data_for_gantt_plotly(
@@ -598,7 +662,7 @@ class BaseProduct(object, metaclass=abc.ABCMeta):
         **kwds,
     ):
         """
-        Draw networkx
+        Draw networkx.
 
         Args:
             G (networkx.Digraph, optional):
@@ -627,21 +691,27 @@ class BaseProduct(object, metaclass=abc.ABCMeta):
         Returns:
             figure: Figure for a network
         """
-        plt.figure(figsize=figsize, dpi=dpi)
+        fig = plt.figure(figsize=figsize, dpi=dpi)
         G = G if G is not None else self.get_networkx_graph()
         pos = pos if pos is not None else nx.spring_layout(G)
-        r_nx = nx.draw_networkx(
+
+        # component
+        component_list = self.component_list
+        nx.draw_networkx_nodes(
             G,
-            pos=pos,
-            arrows=arrows,
+            pos,
+            nodelist=component_list,
             node_color=component_node_color,
-            **kwds,
+            # **kwds,
         )
+
+        nx.draw_networkx_labels(G, pos)
+        nx.draw_networkx_edges(G, pos)
         plt.axis("off")
         if save_fig_path is not None:
             plt.savefig(save_fig_path)
         plt.close()
-        return r_nx
+        return fig
 
     def get_node_and_edge_trace_for_plotly_network(
         self, G=None, pos=None, node_size=20, component_node_color="#FF6600"
@@ -676,10 +746,7 @@ class BaseProduct(object, metaclass=abc.ABCMeta):
             text=[],
             mode="markers",
             hoverinfo="text",
-            marker=dict(
-                color=component_node_color,
-                size=node_size,
-            ),
+            marker={"color": component_node_color, "size": node_size},
         )
 
         for node in G.nodes:
@@ -689,7 +756,11 @@ class BaseProduct(object, metaclass=abc.ABCMeta):
             node_trace["text"] = node_trace["text"] + (node,)
 
         edge_trace = go.Scatter(
-            x=[], y=[], line=dict(width=1, color="#888"), hoverinfo="none", mode="lines"
+            x=[],
+            y=[],
+            line={"width": 1, "color": "#888"},
+            hoverinfo="none",
+            mode="lines",
         )
 
         for edge in G.edges:
@@ -711,7 +782,7 @@ class BaseProduct(object, metaclass=abc.ABCMeta):
         save_fig_path=None,
     ):
         """
-        Draw plotly network
+        Draw plotly network.
 
         Args:
             G (networkx.Digraph, optional):
@@ -753,22 +824,22 @@ class BaseProduct(object, metaclass=abc.ABCMeta):
                 #         hovermode='closest',
                 #         margin=dict(b=20,l=5,r=5,t=40),
                 annotations=[
-                    dict(
-                        ax=edge_trace["x"][index * 2],
-                        ay=edge_trace["y"][index * 2],
-                        axref="x",
-                        ayref="y",
-                        x=edge_trace["x"][index * 2 + 1],
-                        y=edge_trace["y"][index * 2 + 1],
-                        xref="x",
-                        yref="y",
-                        showarrow=True,
-                        arrowhead=5,
-                    )
+                    {
+                        "ax": edge_trace["x"][index * 2],
+                        "ay": edge_trace["y"][index * 2],
+                        "axref": "x",
+                        "ayref": "y",
+                        "x": edge_trace["x"][index * 2 + 1],
+                        "y": edge_trace["y"][index * 2 + 1],
+                        "xref": "x",
+                        "yref": "y",
+                        "showarrow": True,
+                        "arrowhead": 5,
+                    }
                     for index in range(0, int(len(edge_trace["x"]) / 2))
                 ],
-                xaxis=dict(showgrid=False, zeroline=False, showticklabels=False),
-                yaxis=dict(showgrid=False, zeroline=False, showticklabels=False),
+                xaxis={"showgrid": False, "zeroline": False, "showticklabels": False},
+                yaxis={"showgrid": False, "zeroline": False, "showticklabels": False},
             ),
         )
         if save_fig_path is not None:
