@@ -1,30 +1,40 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
+"""base_project."""
 
-from abc import ABCMeta
-import json
 import datetime
-import plotly.figure_factory as ff
-import networkx as nx
-import plotly.graph_objects as go
+import itertools
+import json
+import warnings
+from abc import ABCMeta
+from enum import IntEnum
+
 import matplotlib.pyplot as plt
-from .base_product import BaseProduct
+
+import networkx as nx
+
+import plotly.figure_factory as ff
+import plotly.graph_objects as go
+
 from .base_component import BaseComponent
-from .base_workflow import BaseWorkflow
-from .base_task import BaseTask, BaseTaskState, BaseTaskDependency
+from .base_facility import BaseFacility, BaseFacilityState
 from .base_organization import BaseOrganization
+from .base_priority_rule import (
+    TaskPriorityRuleMode,
+    sort_resource_list,
+    sort_task_list,
+    sort_workplace_list,
+)
+from .base_product import BaseProduct
+from .base_task import BaseTask, BaseTaskDependency, BaseTaskState
 from .base_team import BaseTeam
 from .base_worker import BaseWorker, BaseWorkerState
-from .base_priority_rule import TaskPriorityRule, ResourcePriorityRule
-from enum import IntEnum
-import itertools
+from .base_workflow import BaseWorkflow
 from .base_workplace import BaseWorkplace
-from .base_facility import BaseFacility, BaseFacilityState
-import warnings
 
 
 class SimulationMode(IntEnum):
-    """SimulationMode"""
+    """SimulationMode."""
 
     NONE = 0
     FOWARD = 1
@@ -32,7 +42,8 @@ class SimulationMode(IntEnum):
 
 
 class BaseProject(object, metaclass=ABCMeta):
-    """BaseProject
+    """BaseProject.
+
     BaseProject class for expressing target project
     including product, organization and workflow.
     This class will be used as template.
@@ -88,7 +99,7 @@ class BaseProject(object, metaclass=ABCMeta):
         simulation_mode=None,
         log_txt=None,
     ):
-
+        """init."""
         # ----
         # Constraint parameter on simulation
         # --
@@ -141,7 +152,8 @@ class BaseProject(object, metaclass=ABCMeta):
             self.log_txt = []
 
     def __str__(self):
-        """
+        """str.
+
         Returns:
             str: time and name lists of product, organization and workflow.
         """
@@ -152,6 +164,7 @@ class BaseProject(object, metaclass=ABCMeta):
     def initialize(self, state_info=True, log_info=True):
         """
         Initialize the following changeable variables of BaseProject.
+
         If `state_info` is True, the following attributes are initialized.
 
           - `time`
@@ -187,7 +200,7 @@ class BaseProject(object, metaclass=ABCMeta):
     def simulate(
         self,
         task_performed_mode="multi-workers",
-        task_priority_rule=TaskPriorityRule.TSLACK,
+        task_priority_rule=TaskPriorityRuleMode.TSLACK,
         error_tol=1e-10,
         print_debug=False,
         weekend_working=True,
@@ -199,7 +212,7 @@ class BaseProject(object, metaclass=ABCMeta):
         unit_time=1,
     ):
         """
-        Simulation function for simulate this BaseProject.
+        Simulate this BaseProject.
 
         Args:
             task_performed_mode (str, optional):
@@ -240,7 +253,6 @@ class BaseProject(object, metaclass=ABCMeta):
                 Unit time of simulation.
                 Defaults to 1.
         """
-
         if not (task_performed_mode == "multi-workers"):
             raise Exception(
                 "Please check "
@@ -305,12 +317,7 @@ class BaseProject(object, metaclass=ABCMeta):
 
             # 2. Allocate free workers to READY tasks
             if working:
-                self.__allocate_component_to_workplace(
-                    task_priority_rule=task_priority_rule,
-                    print_debug=print_debug,
-                    log_txt=log_txt_this_time,
-                )
-                self.__allocate_single_task_workers(
+                self.__allocate(
                     task_priority_rule=task_priority_rule,
                     print_debug=print_debug,
                     log_txt=log_txt_this_time,
@@ -341,7 +348,7 @@ class BaseProject(object, metaclass=ABCMeta):
     def backward_simulate(
         self,
         task_performed_mode="multi-workers",
-        task_priority_rule=TaskPriorityRule.TSLACK,
+        task_priority_rule=TaskPriorityRuleMode.TSLACK,
         error_tol=1e-10,
         print_debug=False,
         weekend_working=True,
@@ -355,7 +362,7 @@ class BaseProject(object, metaclass=ABCMeta):
         reverse_log_information=True,
     ):
         """
-        Backward Simulation function for simulate this BaseProject.
+        Simulate this BaseProject by using backward simulation.
 
         Args:
             task_performed_mode (str, optional):
@@ -410,10 +417,10 @@ class BaseProject(object, metaclass=ABCMeta):
             This function is only for research and still in progress.
             Especially, this function is not suitable for simulation considering rework.
         """
-
         self.workflow.reverse_dependencies()
 
         autotask_removing_after_simulation = []
+        success_simulation = False
         try:
             if considering_due_time_of_tail_tasks:
                 # Add dummy task for considering the difference of due_time
@@ -450,9 +457,7 @@ class BaseProject(object, metaclass=ABCMeta):
                 max_time=max_time,
                 unit_time=unit_time,
             )
-
-            if reverse_log_information:
-                self.reverse_log_information()
+            success_simulation = True
 
         finally:
             self.simulation_mode = SimulationMode.BACKWARD
@@ -460,17 +465,24 @@ class BaseProject(object, metaclass=ABCMeta):
                 for task, dependency in autotask.output_task_list:
                     task.input_task_list.remove([autotask, dependency])
                 self.workflow.task_list.remove(autotask)
+            if reverse_log_information:
+                self.reverse_log_information(success_simulation)
             self.workflow.reverse_dependencies()
 
-    def reverse_log_information(self):
-        """
-        Reverse log information of all.
-        """
+    def reverse_log_information(self, delete_head=False):
+        """Reverse log information of all."""
         self.cost_list = self.cost_list[::-1]
         self.log_txt = self.log_txt[::-1]
-        self.product.reverse_log_information()
-        self.organization.reverse_log_information()
-        self.workflow.reverse_log_information()
+        self.product.reverse_log_information(delete_head)
+        self.organization.reverse_log_information(delete_head)
+        self.workflow.reverse_log_information(delete_head)
+        if delete_head:
+            self.cost_list.pop(0)
+            # cost_head = self.cost_list.pop(0)
+            # self.cost_list.append(cost_head)  # insert
+            self.log_txt.pop(0)
+            # log_head = self.log_txt.pop(0)
+            # self.log_txt.append(log_head)  # insert
 
     def __perform(self, print_debug=False, log_txt=[]):
 
@@ -546,16 +558,35 @@ class BaseProject(object, metaclass=ABCMeta):
             print("UPDATE")
         self.workflow.check_state(self.time, BaseTaskState.FINISHED)
         self.product.check_state()  # product should be checked after checking workflow state
-        self.product.check_removing_placed_workplace(
-            print_debug=print_debug, log_txt=log_txt
+        remove_txt = self.product.check_removing_placed_workplace(
+            print_debug=print_debug
         )
+        if len(remove_txt) > 0:
+            log_txt.extend(remove_txt)
         self.workflow.check_state(self.time, BaseTaskState.READY)
         self.workflow.update_PERT_data(self.time)
 
-    def __allocate_component_to_workplace(
-        self, task_priority_rule=TaskPriorityRule.TSLACK, print_debug=False, log_txt=[]
-    ):
+    def __is_allocated_worker(self, worker, task):
+        team = list(
+            filter(lambda team: team.ID == worker.team_id, self.organization.team_list)
+        )[0]
+        return task in team.targeted_task_list
 
+    def __is_allocated_facility(self, facility, task):
+        workplace = list(
+            filter(
+                lambda workplace: workplace.ID == facility.workplace_id,
+                self.organization.workplace_list,
+            )
+        )[0]
+        return task in workplace.targeted_task_list
+
+    def __allocate(
+        self,
+        task_priority_rule=TaskPriorityRuleMode.TSLACK,
+        print_debug=False,
+        log_txt=[],
+    ):
         # LOG: Check free workplace before setting components
         log_txt.append("ALLOCATE")
         log_txt.append("Workplace - Component before setting components in this time")
@@ -565,82 +596,15 @@ class BaseProject(object, metaclass=ABCMeta):
                 + ":"
                 + ",".join([c.name for c in workplace.placed_component_list])
             )
-        if print_debug:
-            print("ALLOCATE")
-            print("Workplace - Component before setting components in this time")
-            for workplace in self.organization.workplace_list:
-                print(
-                    workplace.name
-                    + ":"
-                    + ",".join([c.name for c in workplace.placed_component_list])
-                )
-
-        target_workplace_id_list = [f.ID for f in self.organization.workplace_list]
-
-        # 1. Extract READY components
         ready_component_list = list(
             filter(lambda c: c.is_ready() is True, self.product.component_list)
         )
-
         log_txt.append("Ready Component list before allocating")
         log_txt.append(",".join([c.name for c in ready_component_list]))
         if print_debug:
             print("Ready Component list before allocating")
             print(",".join([c.name for c in ready_component_list]))
 
-        # 2. Decide which workplace put each ready component
-        for ready_component in ready_component_list:
-            ready_task_list = list(
-                filter(
-                    lambda task: task.state == BaseTaskState.READY,
-                    ready_component.targeted_task_list,
-                )
-            )
-
-            # Sort tasks
-            ready_task_list = self.__sort_task(ready_task_list, task_priority_rule)
-
-            for ready_task in ready_task_list:
-                for workplace in ready_task.allocated_workplace_list:
-                    if workplace.ID in target_workplace_id_list:
-                        if (
-                            workplace.can_put(ready_component)
-                            and workplace.get_total_workamount_skill(ready_task.name)
-                            > 1e-10
-                        ):
-                            # move ready_component from None to workplace
-                            pre_workplace = ready_component.placed_workplace
-                            if pre_workplace is not None:
-                                ready_component.set_placed_workplace(None)
-                                pre_workplace.remove_placed_component(ready_component)
-                            ready_component.set_placed_workplace(workplace)
-                            workplace.set_placed_component(ready_component)
-                            break
-
-        # LOG: Check free workplace after setting components
-        log_txt.append("Workplace - Component after setting components in this time")
-        for workplace in self.organization.workplace_list:
-            log_txt.append(
-                workplace.name
-                + ":"
-                + ",".join([c.name for c in workplace.placed_component_list])
-            )
-        if print_debug:
-            print("Workplace - Component after setting components in this time")
-            for workplace in self.organization.workplace_list:
-                print(
-                    workplace.name
-                    + ":"
-                    + ",".join([c.name for c in workplace.placed_component_list])
-                )
-
-    def __allocate_single_task_workers(
-        self,
-        task_priority_rule=TaskPriorityRule.TSLACK,
-        print_debug=False,
-        log_txt=[],
-    ):
-
         # 1. Get ready task and free workers and facilities
         ready_and_working_task_list = list(
             filter(
@@ -675,16 +639,68 @@ class BaseProject(object, metaclass=ABCMeta):
         )
 
         # 2. Sort ready task using TaskPriorityRule
-        ready_and_working_task_list = self.__sort_task(
+        ready_and_working_task_list = sort_task_list(
             ready_and_working_task_list, task_priority_rule
         )
 
         # 3. Allocate ready tasks to free workers and facilities
-        for task in ready_and_working_task_list:
+        target_workplace_id_list = [wp.ID for wp in self.organization.workplace_list]
 
+        for task in ready_and_working_task_list:
+            # 3-1. Set target component of workplace if target component is ready
+            component = task.target_component
+            if component.is_ready():
+                candidate_workplace_list = task.allocated_workplace_list
+                candidate_workplace_list = sort_workplace_list(
+                    candidate_workplace_list,
+                    task.workplace_priority_rule,
+                    name=task.name,
+                )
+                for workplace in candidate_workplace_list:
+                    if workplace.ID in target_workplace_id_list:
+                        conveyor_condition = True
+                        if len(workplace.input_workplace_list) > 0:
+                            if component.placed_workplace is None:
+                                conveyor_condition = True
+                            elif not (
+                                component.placed_workplace
+                                in workplace.input_workplace_list
+                            ):
+                                conveyor_condition = False
+
+                        if (
+                            conveyor_condition
+                            and workplace.can_put(component)
+                            and workplace.get_total_workamount_skill(task.name) > 1e-10
+                        ):
+                            # 3-1-1. move ready_component
+                            pre_workplace = component.placed_workplace
+
+                            # 3-1-1-1. remove
+                            if pre_workplace is None:
+                                for child_c in component.child_component_list:
+                                    wp = child_c.placed_workplace
+                                    if wp is not None:
+                                        for c_wp in wp.placed_component_list:
+                                            if task.target_component.ID in [
+                                                c.ID for c in c_wp.parent_component_list
+                                            ]:
+                                                wp.remove_placed_component(c_wp)
+
+                            elif pre_workplace is not None:
+                                pre_workplace.remove_placed_component(component)
+
+                            component.set_placed_workplace(None)
+
+                            # 3-1-1-2. regsister
+                            component.set_placed_workplace(workplace)
+                            workplace.set_placed_component(component)
+                            break
+
+            # 3-2. Allocate ready tasks to free workers and facilities
             # Worker sorting
-            free_worker_list = self.__sort_resource(
-                free_worker_list, task.worker_priority_rule
+            free_worker_list = sort_resource_list(
+                free_worker_list, task.worker_priority_rule, name=task.name
             )
 
             allocating_workers = list(
@@ -710,7 +726,7 @@ class BaseProject(object, metaclass=ABCMeta):
                     )
 
                     # Facility sorting
-                    free_facility_list = self.__sort_resource(
+                    free_facility_list = sort_resource_list(
                         free_facility_list, task.facility_priority_rule
                     )
 
@@ -727,58 +743,45 @@ class BaseProject(object, metaclass=ABCMeta):
                         for worker in allocating_workers:
                             if task.can_add_resources(worker=worker, facility=facility):
                                 task.allocated_worker_list.append(worker)
+                                worker.assigned_task_list.append(task)
                                 task.allocated_facility_list.append(facility)
+                                facility.assigned_task_list.append(task)
                                 allocating_workers.remove(worker)
                                 free_worker_list = [
                                     w for w in free_worker_list if w.ID != worker.ID
                                 ]
-                                allocating_facilities.remove(facility)
                                 break
 
             else:
                 for worker in allocating_workers:
                     if task.can_add_resources(worker=worker):
                         task.allocated_worker_list.append(worker)
-                        free_worker_list.remove(worker)
+                        worker.assigned_task_list.append(task)
+                        free_worker_list = [
+                            w for w in free_worker_list if w.ID != worker.ID
+                        ]
+
+        # LOG: Check free workplace after setting components
+        log_txt.append("Workplace - Component after setting components in this time")
+        for workplace in self.organization.workplace_list:
+            log_txt.append(
+                workplace.name
+                + ":"
+                + ",".join([c.name for c in workplace.placed_component_list])
+            )
+        if print_debug:
+            print("Workplace - Component after setting components in this time")
+            for workplace in self.organization.workplace_list:
+                print(
+                    workplace.name
+                    + ":"
+                    + ",".join([c.name for c in workplace.placed_component_list])
+                )
 
         # 4. Update state of task newly allocated workers and facilities (READY -> WORKING)
         self.workflow.check_state(self.time, BaseTaskState.WORKING)
         self.product.check_state()  # product should be checked after checking workflow state
 
-    def __is_allocated_worker(self, worker, task):
-        team = list(
-            filter(lambda team: team.ID == worker.team_id, self.organization.team_list)
-        )[0]
-        return task in team.targeted_task_list
-
-    def __is_allocated_facility(self, facility, task):
-        workplace = list(
-            filter(
-                lambda workplace: workplace.ID == facility.workplace_id,
-                self.organization.workplace_list,
-            )
-        )[0]
-        return task in workplace.targeted_task_list
-
-    def __sort_task(self, task_list, priority_rule):
-
-        if priority_rule == TaskPriorityRule.TSLACK:
-            # Task: TSLACK (a task which Slack time(LS-ES) is lower has high priority)
-            task_list = sorted(task_list, key=lambda task: task.lst - task.est)
-
-        return task_list
-
-    def __sort_resource(self, resource_list, priority_rule):
-
-        if priority_rule == ResourcePriorityRule.SSP:
-            # SSP: a resource which amount of skillpoint is lower has high priority
-            resource_list = sorted(
-                resource_list,
-                key=lambda resource: sum(resource.workamount_skill_mean_map.values()),
-            )
-
-        return resource_list
-
     def is_business_time(
         self,
         target_datetime: datetime.datetime,
@@ -845,7 +848,8 @@ class BaseProject(object, metaclass=ABCMeta):
         save_fig_path=None,
     ):
         """
-        Method for creating Gantt chart by plotly.
+        Create Gantt chart by plotly.
+
         This method will be used after simulation.
 
         Args:
@@ -893,12 +897,12 @@ class BaseProject(object, metaclass=ABCMeta):
         colors = (
             colors
             if colors is not None
-            else dict(
-                Component="rgb(246, 37, 105)",
-                Task="rgb(146, 237, 5)",
-                Worker="rgb(46, 137, 205)",
-                Facility="rgb(46, 137, 205)",
-            )
+            else {
+                "Component": "rgb(246, 37, 105)",
+                "Task": "rgb(146, 237, 5)",
+                "Worker": "rgb(46, 137, 205)",
+                "Facility": "rgb(46, 137, 205)",
+            }
         )
         index_col = index_col if index_col is not None else "Type"
         df = []
@@ -1025,7 +1029,7 @@ class BaseProject(object, metaclass=ABCMeta):
         **kwds,
     ):
         """
-        Draw networkx
+        Draw networkx.
 
         Args:
             G (networkx.SDigraph, optional):
@@ -1078,7 +1082,6 @@ class BaseProject(object, metaclass=ABCMeta):
         Returns:
             figure: Figure for a network
         """
-
         fig = plt.figure(figsize=figsize, dpi=dpi)
         G = (
             G
@@ -1246,10 +1249,10 @@ class BaseProject(object, metaclass=ABCMeta):
             text=[],
             mode="markers",
             hoverinfo="text",
-            marker=dict(
-                color=component_node_color,
-                size=node_size,
-            ),
+            marker={
+                "color": component_node_color,
+                "size": node_size,
+            },
         )
 
         task_node_trace = go.Scatter(
@@ -1258,10 +1261,10 @@ class BaseProject(object, metaclass=ABCMeta):
             text=[],
             mode="markers",
             hoverinfo="text",
-            marker=dict(
-                color=task_node_color,
-                size=node_size,
-            ),
+            marker={
+                "color": task_node_color,
+                "size": node_size,
+            },
         )
 
         auto_task_node_trace = go.Scatter(
@@ -1270,10 +1273,10 @@ class BaseProject(object, metaclass=ABCMeta):
             text=[],
             mode="markers",
             hoverinfo="text",
-            marker=dict(
-                color=auto_task_node_color,
-                size=node_size,
-            ),
+            marker={
+                "color": auto_task_node_color,
+                "size": node_size,
+            },
         )
 
         team_node_trace = go.Scatter(
@@ -1282,10 +1285,10 @@ class BaseProject(object, metaclass=ABCMeta):
             text=[],
             mode="markers",
             hoverinfo="text",
-            marker=dict(
-                color=team_node_color,
-                size=node_size,
-            ),
+            marker={
+                "color": team_node_color,
+                "size": node_size,
+            },
         )
 
         worker_node_trace = go.Scatter(
@@ -1294,10 +1297,10 @@ class BaseProject(object, metaclass=ABCMeta):
             text=[],
             mode="markers",
             hoverinfo="text",
-            marker=dict(
-                color=worker_node_color,
-                size=node_size,
-            ),
+            marker={
+                "color": worker_node_color,
+                "size": node_size,
+            },
         )
 
         workplace_node_trace = go.Scatter(
@@ -1306,10 +1309,10 @@ class BaseProject(object, metaclass=ABCMeta):
             text=[],
             mode="markers",
             hoverinfo="text",
-            marker=dict(
-                color=workplace_node_color,
-                size=node_size,
-            ),
+            marker={
+                "color": workplace_node_color,
+                "size": node_size,
+            },
         )
 
         facility_node_trace = go.Scatter(
@@ -1318,14 +1321,18 @@ class BaseProject(object, metaclass=ABCMeta):
             text=[],
             mode="markers",
             hoverinfo="text",
-            marker=dict(
-                color=facility_node_color,
-                size=node_size,
-            ),
+            marker={
+                "color": facility_node_color,
+                "size": node_size,
+            },
         )
 
         edge_trace = go.Scatter(
-            x=[], y=[], line=dict(width=0, color="#888"), hoverinfo="none", mode="lines"
+            x=[],
+            y=[],
+            line={"width": 0, "color": "#888"},
+            hoverinfo="none",
+            mode="lines",
         )
 
         for node in G.nodes:
@@ -1399,7 +1406,7 @@ class BaseProject(object, metaclass=ABCMeta):
         save_fig_path=None,
     ):
         """
-        Draw plotly network
+        Draw plotly network.
 
         Args:
             G (networkx.Digraph, optional):
@@ -1487,22 +1494,22 @@ class BaseProject(object, metaclass=ABCMeta):
                 #         hovermode='closest',
                 #         margin=dict(b=20,l=5,r=5,t=40),
                 annotations=[
-                    dict(
-                        ax=edge_trace["x"][index * 2],
-                        ay=edge_trace["y"][index * 2],
-                        axref="x",
-                        ayref="y",
-                        x=edge_trace["x"][index * 2 + 1],
-                        y=edge_trace["y"][index * 2 + 1],
-                        xref="x",
-                        yref="y",
-                        showarrow=True,
-                        arrowhead=5,
-                    )
+                    {
+                        "ax": edge_trace["x"][index * 2],
+                        "ay": edge_trace["y"][index * 2],
+                        "axref": "x",
+                        "ayref": "y",
+                        "x": edge_trace["x"][index * 2 + 1],
+                        "y": edge_trace["y"][index * 2 + 1],
+                        "xref": "x",
+                        "yref": "y",
+                        "showarrow": True,
+                        "arrowhead": 5,
+                    }
                     for index in range(0, int(len(edge_trace["x"]) / 2))
                 ],
-                xaxis=dict(showgrid=False, zeroline=False, showticklabels=False),
-                yaxis=dict(showgrid=False, zeroline=False, showticklabels=False),
+                xaxis={"showgrid": False, "zeroline": False, "showticklabels": False},
+                yaxis={"showgrid": False, "zeroline": False, "showticklabels": False},
             ),
         )
         if save_fig_path is not None:
@@ -1535,6 +1542,7 @@ class BaseProject(object, metaclass=ABCMeta):
     def output_simlog(self, file_path):
         """
         Create simulation log text file.
+
         Args:
             file_path (str):
                 File path for saving simulation log.
@@ -1546,6 +1554,7 @@ class BaseProject(object, metaclass=ABCMeta):
     def write_simple_json(self, file_path, encoding="utf-8", indent=4):
         """
         Create json file of this project.
+
         Args:
             file_path (str):
                 File path for saving this project data.
