@@ -1,32 +1,39 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
+"""base_resource."""
 
-from debtcollector import removals
 import abc
 import uuid
+import warnings
 from enum import IntEnum
+
 import numpy as np
+
 from .base_task import BaseTaskState
 
 
-@removals.remove
 class BaseResourceState(IntEnum):
-    """
+    """BaseResourceState.
+
     Note:
-        DEPRICATED from v1.3.
+        DEPRICATED from v0.3.
         Use BaseWorkerState or BaseFacilityState as instead.
     """
 
+    warnings.warn(
+        "DEPRICATED from v1.3. Use BaseWorkerState or BaseFacilityState as instead.",
+        DeprecationWarning,
+    )
+
     FREE = 0
     WORKING = 1
 
 
-@removals.remove
 class BaseResource(object, metaclass=abc.ABCMeta):
-    """BaseResource
+    """BaseResource.
 
     Note:
-        DEPRICATED from v1.3.
+        DEPRICATED from v0.3.
         Use BaseWorkerState or BaseFacilityState as instead.
 
 
@@ -82,6 +89,11 @@ class BaseResource(object, metaclass=abc.ABCMeta):
             Defaults to None -> [].
     """
 
+    warnings.warn(
+        "DEPRICATED from v1.3. Use BaseWorker or BaseFacility as instead.",
+        DeprecationWarning,
+    )
+
     def __init__(
         self,
         # Basic parameters
@@ -99,7 +111,7 @@ class BaseResource(object, metaclass=abc.ABCMeta):
         assigned_task_list=None,
         assigned_task_id_record=None,
     ):
-
+        """init."""
         # ----
         # Constraint parameter on simulation
         # --
@@ -110,7 +122,7 @@ class BaseResource(object, metaclass=abc.ABCMeta):
         self.cost_per_time = cost_per_time if cost_per_time != 0.0 else 0.0
         self.solo_working = solo_working if solo_working is not None else False
         self.workamount_skill_mean_map = (
-            workamount_skill_mean_map if workamount_skill_mean_map is not {} else {}
+            workamount_skill_mean_map if workamount_skill_mean_map != {} else {}
         )
         self.workamount_skill_sd_map = (
             workamount_skill_sd_map if workamount_skill_sd_map is not None else {}
@@ -146,7 +158,8 @@ class BaseResource(object, metaclass=abc.ABCMeta):
             self.assigned_task_id_record = []
 
     def __str__(self):
-        """
+        """str.
+
         Returns:
             str: name of BaseResource
         Examples:
@@ -159,6 +172,7 @@ class BaseResource(object, metaclass=abc.ABCMeta):
     def initialize(self, state_info=True, log_info=True):
         """
         Initialize the following changeable variables of BaseResource.
+
         If `state_info` is True, the following attributes are initialized.
 
           - `state`
@@ -187,26 +201,30 @@ class BaseResource(object, metaclass=abc.ABCMeta):
             self.cost_list = []
             self.assigned_task_id_record = []
 
-    def reverse_log_information(self):
-        """
-        Reverse log information of all.
-        """
+    def reverse_log_information(self, delete_head=False):
+        """Reverse log information of all."""
         self.state_record_list = self.state_record_list[::-1]
         self.cost_list = self.cost_list[::-1]
         self.assigned_task_id_record = self.assigned_task_id_record[::-1]
+        if delete_head:
+            self.cost_list.pop(0)
+            # cost_head = self.cost_list.pop(0)
+            # self.cost_list.append(cost_head)  # insert
+            self.state_record_list.pop(0)
+            # log_head = self.state_record_list.pop(0)
+            # self.state_record_list.append(log_head)  # insert
+            self.assigned_task_id_record.pop(0)
+            # log_head = self.assigned_task_id_record.pop(0)
+            # self.assigned_task_id_record.append(log_head)  # insert
 
     def record_assigned_task_id(self):
-        """
-        Record assigned task id to `assigned_task_id_record`.
-        """
+        """Record assigned task id to `assigned_task_id_record`."""
         self.assigned_task_id_record.append(
             [task.ID for task in self.assigned_task_list]
         )
 
     def record_state(self):
-        """
-        Record current 'state' in 'state_record_list'
-        """
+        """Record current 'state' in 'state_record_list'."""
         self.state_record_list.append(self.state)
 
     def get_time_list_for_gannt_chart(self, finish_margin=1.0):
@@ -247,9 +265,7 @@ class BaseResource(object, metaclass=abc.ABCMeta):
             previous_state = state
 
             if previous_state == BaseResourceState.WORKING:
-                working_time_list.append(
-                    (from_time, time - 1 - from_time + finish_margin)
-                )
+                working_time_list.append((from_time, time - from_time + finish_margin))
             elif previous_state == BaseResourceState.FREE:
                 ready_time_list.append(
                     (from_time, time - 1 - from_time + finish_margin)
@@ -258,8 +274,9 @@ class BaseResource(object, metaclass=abc.ABCMeta):
 
     def has_workamount_skill(self, task_name, error_tol=1e-10):
         """
-        Check whether he or she has workamount skill or not
-        by checking workamount_skill_mean_map.
+        Check whether he or she has workamount skill or not.
+
+        By checking workamount_skill_mean_map.
 
         Args:
             task_name (str):
