@@ -1,15 +1,17 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
+"""base_task."""
 
 import abc
+import datetime
 import uuid
 from enum import IntEnum
-import datetime
-from .base_priority_rule import ResourcePriorityRule
+
+from .base_priority_rule import ResourcePriorityRuleMode, WorkplacePriorityRuleMode
 
 
 class BaseTaskState(IntEnum):
-    """BaseTaskState"""
+    """BaseTaskState."""
 
     NONE = 0
     READY = 1
@@ -19,7 +21,7 @@ class BaseTaskState(IntEnum):
 
 
 class BaseTaskDependency(IntEnum):
-    """BaseTaskDependency"""
+    """BaseTaskDependency."""
 
     FS = 0  # Finish to Start
     SS = 1  # Start to Start
@@ -28,7 +30,8 @@ class BaseTaskDependency(IntEnum):
 
 
 class BaseTask(object, metaclass=abc.ABCMeta):
-    """BaseTask
+    """BaseTask.
+
     BaseTask class for expressing target workflow.
     This class will be used as template.
 
@@ -58,8 +61,15 @@ class BaseTask(object, metaclass=abc.ABCMeta):
             Defaults to None -> [].
         allocated_workplace_list (List[BaseWorkplace], optional):
             Basic parameter.
-            List of allocated BaseWorkplace
+            List of allocated BaseWorkplace.
             Defaults to None -> [].
+        parent_workflow (BaseWorkflow, optional):
+            Basic parameter.
+            Parent workflow.
+            Defaults to None.
+        workplace_priority_rule (WorkplacePriorityRuleMode, oprional):
+            Workplace priority rule for simulation.
+            Deraults to WorkplacePriorityRuleMode.FSS.
         worker_priority_rule (ResourcePriorityRule, oprional):
             Worker priority rule for simulation.
             Deraults to ResourcePriorityRule.SSP.
@@ -150,8 +160,10 @@ class BaseTask(object, metaclass=abc.ABCMeta):
         output_task_list=None,
         allocated_team_list=None,
         allocated_workplace_list=None,
-        worker_priority_rule=ResourcePriorityRule.SSP,
-        facility_priority_rule=ResourcePriorityRule.SSP,
+        parent_workflow=None,
+        workplace_priority_rule=WorkplacePriorityRuleMode.FSS,
+        worker_priority_rule=ResourcePriorityRuleMode.SSP,
+        facility_priority_rule=ResourcePriorityRuleMode.SSP,
         need_facility=False,
         target_component=None,
         default_progress=None,
@@ -172,7 +184,7 @@ class BaseTask(object, metaclass=abc.ABCMeta):
         allocated_facility_list=None,
         allocated_facility_id_record=None,
     ):
-
+        """init."""
         # ----
         # Constraint parameter on simulation
         # --
@@ -190,15 +202,21 @@ class BaseTask(object, metaclass=abc.ABCMeta):
         self.allocated_workplace_list = (
             allocated_workplace_list if allocated_workplace_list is not None else []
         )
+        self.parent_workflow = parent_workflow if parent_workflow is not None else None
+        self.workplace_priority_rule = (
+            workplace_priority_rule
+            if workplace_priority_rule is not None
+            else WorkplacePriorityRuleMode.FSS
+        )
         self.worker_priority_rule = (
             worker_priority_rule
             if worker_priority_rule is not None
-            else ResourcePriorityRule.SSP
+            else ResourcePriorityRuleMode.SSP
         )
         self.facility_priority_rule = (
             facility_priority_rule
             if facility_priority_rule is not None
-            else ResourcePriorityRule.SSP
+            else ResourcePriorityRuleMode.SSP
         )
         self.need_facility = need_facility
         self.target_component = (
@@ -265,7 +283,8 @@ class BaseTask(object, metaclass=abc.ABCMeta):
             self.allocated_facility_id_record = []
 
     def __str__(self):
-        """
+        """str.
+
         Returns:
             str: name of BaseTask
         Examples:
@@ -325,7 +344,7 @@ class BaseTask(object, metaclass=abc.ABCMeta):
 
     def append_input_task(self, input_task, task_dependency_mode=BaseTaskDependency.FS):
         """
-        Append input task to `input_task_list`
+        Append input task to `input_task_list`.
 
         Args:
             input_task (BaseTask):
@@ -344,7 +363,6 @@ class BaseTask(object, metaclass=abc.ABCMeta):
             >>> print([parent_t.name for parent_t in task1.output_task_list])
             ['task']
         """
-
         self.input_task_list.append([input_task, task_dependency_mode])
         input_task.output_task_list.append([self, task_dependency_mode])
 
@@ -352,7 +370,7 @@ class BaseTask(object, metaclass=abc.ABCMeta):
         self, input_task_list, task_dependency_mode=BaseTaskDependency.FS
     ):
         """
-        Extend the list of input tasks to `input_task_list`
+        Extend the list of input tasks to `input_task_list`.
 
         Args:
             input_task_list (List[BaseTask]):
@@ -377,6 +395,7 @@ class BaseTask(object, metaclass=abc.ABCMeta):
     def initialize(self, error_tol=1e-10, state_info=True, log_info=True):
         """
         Initialize the following changeable variables of BaseTask.
+
         If `state_info` is True, the following attributes are initialized.
 
           - `est`
@@ -431,7 +450,7 @@ class BaseTask(object, metaclass=abc.ABCMeta):
 
     def perform(self, time: int, seed=None):
         """
-        Perform this BaseTask in this simulation
+        Perform this BaseTask in this simulation.
 
         Args:
             time (int):
@@ -518,6 +537,11 @@ class BaseTask(object, metaclass=abc.ABCMeta):
                 if facility.ID not in self.fixing_allocating_facility_id_list:
                     return False
 
+        # multi-task in one facility check
+        if facility is not None:
+            if len(facility.assigned_task_list) > 0:
+                return False
+
         # skill check
         if facility is not None:
             if facility.has_workamount_skill(self.name):
@@ -539,7 +563,9 @@ class BaseTask(object, metaclass=abc.ABCMeta):
 
     def record_allocated_workers_facilities_id(self):
         """
-        Record allocated worker & facilities id to `allocated_worker_id_record` and `allocated_facility_id_record`.
+        Record allocated worker & facilities id.
+
+        Target attributes are `allocated_worker_id_record` and `allocated_facility_id_record`.
         """
         self.allocated_worker_id_record.append(
             [worker.ID for worker in self.allocated_worker_list]
@@ -549,22 +575,28 @@ class BaseTask(object, metaclass=abc.ABCMeta):
         )
 
     def record_state(self):
-        """
-        Record current 'state' in 'state_record_list'
-        """
+        """Record current 'state' in 'state_record_list'."""
         self.state_record_list.append(self.state)
 
-    def reverse_log_information(self):
-        """
-        Reverse log information of all.
-        """
+    def reverse_log_information(self, delete_head=False):
+        """Reverse log information of all."""
         self.state_record_list = self.state_record_list[::-1]
         self.allocated_worker_id_record = self.allocated_worker_id_record[::-1]
         self.allocated_facility_id_record = self.allocated_facility_id_record[::-1]
+        if delete_head:
+            self.state_record_list.pop(0)
+            # cost_head = self.state_record_list.pop(0)
+            # self.state_record_list.append(cost_head)  # insert
+            self.allocated_worker_id_record.pop(0)
+            # log_head = self.allocated_worker_id_record.pop(0)
+            # self.allocated_worker_id_record.append(log_head)  # insert
+            self.allocated_facility_id_record.pop(0)
+            # log_head = self.allocated_facility_id_record.pop(0)
+            # self.allocated_facility_id_record.append(log_head)  # insert
 
     def get_state_from_record(self, time: int):
         """
-        Get the state information in time
+        Get the state information in time.
 
         Args:
             time (int):
@@ -615,20 +647,16 @@ class BaseTask(object, metaclass=abc.ABCMeta):
                     if state == BaseTaskState.WORKING:
                         if previous_state == BaseTaskState.READY:
                             ready_time_list.append(
-                                (from_time, (to_time - 1) - from_time + finish_margin)
+                                (from_time, to_time - from_time + finish_margin)
                             )
                     from_time = time
                     to_time = -1
             previous_state = state
 
             if previous_state == BaseTaskState.WORKING:
-                working_time_list.append(
-                    (from_time, time - 1 - from_time + finish_margin)
-                )
+                working_time_list.append((from_time, time - from_time + finish_margin))
             elif previous_state == BaseTaskState.READY:
-                ready_time_list.append(
-                    (from_time, time - 1 - from_time + finish_margin)
-                )
+                ready_time_list.append((from_time, time - from_time + finish_margin))
         return ready_time_list, working_time_list
 
     def create_data_for_gantt_plotly(
@@ -666,31 +694,31 @@ class BaseTask(object, metaclass=abc.ABCMeta):
             for (from_time, length) in ready_time_list:
                 to_time = from_time + length
                 df.append(
-                    dict(
-                        Task=self.name,
-                        Start=(init_datetime + from_time * unit_timedelta).strftime(
+                    {
+                        "Task": self.name,
+                        "Start": (init_datetime + from_time * unit_timedelta).strftime(
                             "%Y-%m-%d %H:%M:%S"
                         ),
-                        Finish=(init_datetime + to_time * unit_timedelta).strftime(
+                        "Finish": (init_datetime + to_time * unit_timedelta).strftime(
                             "%Y-%m-%d %H:%M:%S"
                         ),
-                        State="READY",
-                        Type="Task",
-                    )
+                        "State": "READY",
+                        "Type": "Task",
+                    }
                 )
         for (from_time, length) in working_time_list:
             to_time = from_time + length
             df.append(
-                dict(
-                    Task=self.name,
-                    Start=(init_datetime + from_time * unit_timedelta).strftime(
+                {
+                    "Task": self.name,
+                    "Start": (init_datetime + from_time * unit_timedelta).strftime(
                         "%Y-%m-%d %H:%M:%S"
                     ),
-                    Finish=(init_datetime + to_time * unit_timedelta).strftime(
+                    "Finish": (init_datetime + to_time * unit_timedelta).strftime(
                         "%Y-%m-%d %H:%M:%S"
                     ),
-                    State="WORKING",
-                    Type="Task",
-                )
+                    "State": "WORKING",
+                    "Type": "Task",
+                }
             )
         return df
