@@ -1,21 +1,27 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
+"""base_workflow."""
 
 import abc
-from typing import List
-from .base_task import BaseTask, BaseTaskState, BaseTaskDependency
-from .base_worker import BaseWorkerState
-from .base_facility import BaseFacilityState
-import plotly.figure_factory as ff
-import networkx as nx
-import plotly.graph_objects as go
 import datetime
-import matplotlib.pyplot as plt
 import warnings
+from typing import List
+
+import matplotlib.pyplot as plt
+
+import networkx as nx
+
+import plotly.figure_factory as ff
+import plotly.graph_objects as go
+
+from .base_facility import BaseFacilityState
+from .base_task import BaseTask, BaseTaskDependency, BaseTaskState
+from .base_worker import BaseWorkerState
 
 
 class BaseWorkflow(object, metaclass=abc.ABCMeta):
-    """BaseWorkflow
+    """BaseWorkflow.
+
     BaseWorkflow class for expressing workflow in a project.
     BaseWorkflow is consist of multiple BaseTasks.
     This class will be used as template.
@@ -37,6 +43,7 @@ class BaseWorkflow(object, metaclass=abc.ABCMeta):
         # Basic variables
         critical_path_length=0.0,
     ):
+        """init."""
         # ----
         # Constraint parameter on simulation
         # --
@@ -52,7 +59,8 @@ class BaseWorkflow(object, metaclass=abc.ABCMeta):
         )
 
     def __str__(self):
-        """
+        """str.
+
         Returns:
             str: name list of BaseTask
         Examples:
@@ -238,6 +246,7 @@ class BaseWorkflow(object, metaclass=abc.ABCMeta):
     ):
         """
         Get task list by using search conditions related to BaseTask parameter.
+
         If there is no searching condition, this function returns all `task_list`
 
         Args:
@@ -444,6 +453,7 @@ class BaseWorkflow(object, metaclass=abc.ABCMeta):
     def initialize(self, state_info=True, log_info=True):
         """
         Initialize the changeable variables of BaseWorkflow including PERT calculation.
+
         If `state_info` is True, the following attributes are initialized.
 
           - `critical_path_length`
@@ -462,29 +472,27 @@ class BaseWorkflow(object, metaclass=abc.ABCMeta):
         """
         for task in self.task_list:
             task.initialize(state_info=state_info, log_info=log_info)
+            if task.parent_workflow is None:
+                task.parent_workflow = self
         if state_info:
             self.critical_path_length = 0.0
             self.update_PERT_data(0)
             self.check_state(-1, BaseTaskState.READY)
 
-    def reverse_log_information(self):
-        """
-        Reverse log information of all.
-        """
+    def reverse_log_information(self, delete_head=False):
+        """Reverse log information of all."""
         for t in self.task_list:
-            t.reverse_log_information()
+            t.reverse_log_information(delete_head)
 
     def record(self):
-        """
-        Record the state of all tasks in `task_list`.
-        """
+        """Record the state of all tasks in `task_list`."""
         for task in self.task_list:
             task.record_allocated_workers_facilities_id()
             task.record_state()
 
     def update_PERT_data(self, time: int):
         """
-        Update PERT data (est,eft,lst,lft) of each BaseTask in task_list
+        Update PERT data (est,eft,lst,lft) of each BaseTask in task_list.
 
         Args:
             time (int):
@@ -495,7 +503,7 @@ class BaseWorkflow(object, metaclass=abc.ABCMeta):
 
     def check_state(self, time: int, state: BaseTaskState):
         """
-        Check state of all BaseTasks in task_list
+        Check state of all BaseTasks in task_list.
 
         Args:
             time (int):
@@ -577,22 +585,22 @@ class BaseWorkflow(object, metaclass=abc.ABCMeta):
                 task.state = BaseTaskState.WORKING
                 for worker in task.allocated_worker_list:
                     worker.state = BaseWorkerState.WORKING
-                    worker.assigned_task_list.append(task)
+                    # worker.assigned_task_list.append(task)
                 if task.need_facility:
                     for facility in task.allocated_facility_list:
                         facility.state = BaseFacilityState.WORKING
-                        facility.assigned_task_list.append(task)
+                        # facility.assigned_task_list.append(task)
 
             elif task.state == BaseTaskState.WORKING:
                 for worker in task.allocated_worker_list:
                     if worker.state == BaseWorkerState.FREE:
                         worker.state = BaseWorkerState.WORKING
-                        worker.assigned_task_list.append(task)
+                        # worker.assigned_task_list.append(task)
                     if task.need_facility:
                         for facility in task.allocated_facility_list:
                             if facility.state == BaseFacilityState.FREE:
                                 facility.state = BaseFacilityState.WORKING
-                                facility.assigned_task_list.append(task)
+                                # facility.assigned_task_list.append(task)
 
     def __check_finished(self, time: int, error_tol=1e-10):
         working_and_zero_task_list = list(
@@ -783,14 +791,11 @@ class BaseWorkflow(object, metaclass=abc.ABCMeta):
                 Random seed for describing deviation of progress.
                 If workamount
                 Defaults to None.
-        Note:
-            This method includes advanced code of custom simulation.
-            We have to separete basic code and advanced code in the future.
         """
         for task in self.task_list:
             task.perform(time, seed=seed)
 
-    def create_simple_gantt(
+    def plot_simple_gantt(
         self,
         finish_margin=1.0,
         view_auto_task=False,
@@ -803,8 +808,8 @@ class BaseWorkflow(object, metaclass=abc.ABCMeta):
         save_fig_path=None,
     ):
         """
+        Plot Gantt chart by matplotlib.
 
-        Method for creating Gantt chart by matplotlib.
         In this Gantt chart, datetime information is not included.
         This method will be used after simulation.
 
@@ -839,7 +844,70 @@ class BaseWorkflow(object, metaclass=abc.ABCMeta):
 
         Returns:
             fig: fig in plt.subplots()
-            gnt: gnt in plt.subplots()
+        """
+        fig, gnt = self.create_simple_gantt(
+            finish_margin=finish_margin,
+            view_auto_task=view_auto_task,
+            view_ready=view_ready,
+            task_color=task_color,
+            auto_task_color=auto_task_color,
+            ready_color=ready_color,
+            figsize=figsize,
+            dpi=dpi,
+            save_fig_path=save_fig_path,
+        )
+        return fig
+
+    def create_simple_gantt(
+        self,
+        finish_margin=1.0,
+        view_auto_task=False,
+        view_ready=False,
+        task_color="#00EE00",
+        auto_task_color="#005500",
+        ready_color="#C0C0C0",
+        figsize=[6.4, 4.8],
+        dpi=100.0,
+        save_fig_path=None,
+    ):
+        """
+        Create Gantt chart by matplotlib.
+
+        In this Gantt chart, datetime information is not included.
+        This method will be used after simulation.
+
+        Args:
+            finish_margin (float, optional):
+                Margin of finish time in Gantt chart.
+                Defaults to 1.0.
+            view_auto_task (bool, optional):
+                View auto_task or not.
+                Defaults to False.
+            view_ready (bool, optional):
+                View READY time or not.
+                Defaults to False.
+            task_color (str, optional):
+                Task color setting information.
+                Defaults to "#00EE00".
+            auto_task_color (str, optional):
+                Auto Task color setting information.
+                Defaults to "#005500".
+            ready_color (str, optional):
+                Ready Task color setting information.
+                Defaults to "#C0C0C0".
+            figsize ((float, float), optional):
+                Width, height in inches.
+                Default to [6.4, 4.8]
+            dpi (float, optional):
+                The resolution of the figure in dots-per-inch.
+                Default to 100.0
+            save_fig_path (str, optional):
+                Path of saving figure.
+                Defaults to None.
+
+        Returns:
+            fig: fig in plt.subplots()
+            gnt: ax in plt.subplots()
         """
         fig, gnt = plt.subplots()
         fig.figsize = figsize
@@ -892,8 +960,7 @@ class BaseWorkflow(object, metaclass=abc.ABCMeta):
         view_ready=False,
     ):
         """
-        Create data for gantt plotly
-        from task_list.
+        Create data for gantt plotly from task_list.
 
         Args:
             init_datetime (datetime.datetime):
@@ -938,7 +1005,8 @@ class BaseWorkflow(object, metaclass=abc.ABCMeta):
         save_fig_path=None,
     ):
         """
-        Method for creating Gantt chart by plotly.
+        Create Gantt chart by plotly.
+
         This method will be used after simulation.
 
         Args:
@@ -985,7 +1053,7 @@ class BaseWorkflow(object, metaclass=abc.ABCMeta):
         colors = (
             colors
             if colors is not None
-            else dict(WORKING="rgb(146, 237, 5)", READY="rgb(107, 127, 135)")
+            else {"WORKING": "rgb(146, 237, 5)", "READY": "rgb(107, 127, 135)"}
         )
         index_col = index_col if index_col is not None else "State"
         df = self.create_data_for_gantt_plotly(
@@ -1064,7 +1132,7 @@ class BaseWorkflow(object, metaclass=abc.ABCMeta):
         **kwds,
     ):
         """
-        Draw networkx
+        Draw networkx.
 
         Args:
             G (networkx.Digraph, optional):
@@ -1170,10 +1238,10 @@ class BaseWorkflow(object, metaclass=abc.ABCMeta):
             text=[],
             mode="markers",
             hoverinfo="text",
-            marker=dict(
-                color=task_node_color,
-                size=node_size,
-            ),
+            marker={
+                "color": task_node_color,
+                "size": node_size,
+            },
         )
 
         auto_task_node_trace = go.Scatter(
@@ -1182,10 +1250,10 @@ class BaseWorkflow(object, metaclass=abc.ABCMeta):
             text=[],
             mode="markers",
             hoverinfo="text",
-            marker=dict(
-                color=auto_task_node_color,
-                size=node_size,
-            ),
+            marker={
+                "color": auto_task_node_color,
+                "size": node_size,
+            },
         )
 
         for node in G.nodes:
@@ -1200,7 +1268,11 @@ class BaseWorkflow(object, metaclass=abc.ABCMeta):
                 auto_task_node_trace["text"] = auto_task_node_trace["text"] + (node,)
 
         edge_trace = go.Scatter(
-            x=[], y=[], line=dict(width=1, color="#888"), hoverinfo="none", mode="lines"
+            x=[],
+            y=[],
+            line={"width": 1, "color": "#888"},
+            hoverinfo="none",
+            mode="lines",
         )
 
         for edge in G.edges:
@@ -1224,7 +1296,7 @@ class BaseWorkflow(object, metaclass=abc.ABCMeta):
         save_fig_path=None,
     ):
         """
-        Draw plotly network
+        Draw plotly network.
 
         Args:
             G (networkx.Digraph, optional):
@@ -1277,22 +1349,22 @@ class BaseWorkflow(object, metaclass=abc.ABCMeta):
                 #         hovermode='closest',
                 #         margin=dict(b=20,l=5,r=5,t=40),
                 annotations=[
-                    dict(
-                        ax=edge_trace["x"][index * 2],
-                        ay=edge_trace["y"][index * 2],
-                        axref="x",
-                        ayref="y",
-                        x=edge_trace["x"][index * 2 + 1],
-                        y=edge_trace["y"][index * 2 + 1],
-                        xref="x",
-                        yref="y",
-                        showarrow=True,
-                        arrowhead=5,
-                    )
+                    {
+                        "ax": edge_trace["x"][index * 2],
+                        "ay": edge_trace["y"][index * 2],
+                        "axref": "x",
+                        "ayref": "y",
+                        "x": edge_trace["x"][index * 2 + 1],
+                        "y": edge_trace["y"][index * 2 + 1],
+                        "xref": "x",
+                        "yref": "y",
+                        "showarrow": True,
+                        "arrowhead": 5,
+                    }
                     for index in range(0, int(len(edge_trace["x"]) / 2))
                 ],
-                xaxis=dict(showgrid=False, zeroline=False, showticklabels=False),
-                yaxis=dict(showgrid=False, zeroline=False, showticklabels=False),
+                xaxis={"showgrid": False, "zeroline": False, "showticklabels": False},
+                yaxis={"showgrid": False, "zeroline": False, "showticklabels": False},
             ),
         )
         if save_fig_path is not None:
