@@ -1,13 +1,16 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
+"""organization."""
+
+from typing import List
 
 from .base_organization import BaseOrganization
 from .base_team import BaseTeam
-from typing import List
 
 
 class Organization(BaseOrganization):
-    """Organization
+    """Organization.
+
     Organization class for expressing organization in target project.
     This class is implemented from BaseOrganization.
 
@@ -26,4 +29,5 @@ class Organization(BaseOrganization):
     """
 
     def __init__(self, team_list: List[BaseTeam], workplace_list=None, cost_list=None):
+        """init."""
         super().__init__(team_list, workplace_list=workplace_list, cost_list=cost_list)
