@@ -1,13 +1,16 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
+"""product."""
+
+from typing import List
 
 from .base_component import BaseComponent
 from .base_product import BaseProduct
-from typing import List
 
 
 class Product(BaseProduct):
-    """BaseProduct
+    """BaseProduct.
+
     Product class for expressing target product in a project.
     This class is implemented from BaseProduct.
 
@@ -17,4 +20,5 @@ class Product(BaseProduct):
     """
 
     def __init__(self, component_list: List[BaseComponent]):
+        """init."""
         super().__init__(component_list)
